@@ -25,6 +25,7 @@
 #include "bus.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "activation.h"
 #include "connection.h"
@@ -37,6 +38,7 @@
 #include "apparmor.h"
 #include "audit.h"
 #include "dir-watch.h"
+#include <dbus/dbus-auth.h>
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-hash.h>
 #include <dbus/dbus-credentials.h>
@@ -429,6 +431,26 @@ process_config_first_time_only (BusContext       *context,
       link = _dbus_list_get_first_link (auth_mechanisms_list);
       while (link != NULL)
         {
+          DBusString name;
+          _dbus_string_init_const (&name, link->data);
+          if (!_dbus_auth_is_supported_mechanism (&name))
+            {
+              DBusString list;
+              if (!_dbus_string_init (&list))
+                goto oom;
+
+              if (!_dbus_auth_dump_supported_mechanisms (&list))
+                {
+                  _dbus_string_free (&list);
+                  goto oom;
+                }
+              dbus_set_error (error, DBUS_ERROR_FAILED,
+                              "Unsupported auth mechanism \"%s\" in bus config file detected. Supported mechanisms are \"%s\".",
+                              (char*)link->data,
+                              _dbus_string_get_const_data (&list));
+              _dbus_string_free (&list);
+              goto failed;
+            }
           auth_mechanisms[i] = _dbus_strdup (link->data);
           if (auth_mechanisms[i] == NULL)
             goto oom;
@@ -649,32 +671,6 @@ process_config_every_time (BusContext      *context,
   return retval;
 }
 
-static dbus_bool_t
-list_concat_new (DBusList **a,
-                 DBusList **b,
-                 DBusList **result)
-{
-  DBusList *link;
-
-  *result = NULL;
-
-  for (link = _dbus_list_get_first_link (a); link; link = _dbus_list_get_next_link (a, link))
-    {
-      if (!_dbus_list_append (result, link->data))
-        goto oom;
-    }
-  for (link = _dbus_list_get_first_link (b); link; link = _dbus_list_get_next_link (b, link))
-    {
-      if (!_dbus_list_append (result, link->data))
-        goto oom;
-    }
-
-  return TRUE;
-oom:
-  _dbus_list_clear (result);
-  return FALSE;
-}
-
 static void
 raise_file_descriptor_limit (BusContext      *context)
 {
@@ -735,9 +731,7 @@ process_config_postinit (BusContext      *context,
   /* We need to monitor both the configuration directories and directories
    * containing .service files.
    */
-  if (!list_concat_new (bus_config_parser_get_conf_dirs (parser),
-                        bus_config_parser_get_service_dirs (parser),
-                        &watched_dirs))
+  if (!bus_config_parser_get_watched_dirs (parser, &watched_dirs))
     {
       BUS_SET_OOM (error);
       return FALSE;
@@ -970,7 +964,10 @@ bus_context_new (const DBusString *config_file,
 
   if (!bus_selinux_full_init ())
     {
-      bus_context_log (context, DBUS_SYSTEM_LOG_FATAL, "SELinux enabled but D-Bus initialization failed; check system log\n");
+      bus_context_log (context, DBUS_SYSTEM_LOG_ERROR,
+                       "SELinux enabled but D-Bus initialization failed; "
+                       "check system log");
+      exit (1);
     }
 
   if (!bus_apparmor_full_init (error))
