@@ -203,20 +203,6 @@ bus_config_load (const DBusString      *file,
       goto failed;
     }
 
-  /* We do not need protection against hash collisions (CVE-2012-0876)
-   * because we are only parsing trusted XML; and if we let Expat block
-   * waiting for the CSPRNG to be initialized, as it does by default to
-   * defeat CVE-2012-0876, it can cause timeouts during early boot on
-   * entropy-starved embedded devices.
-   *
-   * TODO: When Expat gets a more explicit API for this than
-   * XML_SetHashSalt, check for that too, and use it preferentially.
-   * https://github.com/libexpat/libexpat/issues/91 */
-#if defined(HAVE_XML_SETHASHSALT)
-  /* Any nonzero number will do. https://xkcd.com/221/ */
-  XML_SetHashSalt (expat, 4);
-#endif
-
   if (!_dbus_string_get_dirname (file, &dirname))
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
@@ -256,7 +242,7 @@ bus_config_load (const DBusString      *file,
 
     data_str = _dbus_string_get_const_data (&data);
 
-    if (!XML_Parse (expat, data_str, _dbus_string_get_length (&data), TRUE))
+    if (XML_Parse (expat, data_str, _dbus_string_get_length (&data), TRUE) == XML_STATUS_ERROR)
       {
         if (context.error != NULL &&
             !dbus_error_is_set (context.error))
@@ -268,10 +254,13 @@ bus_config_load (const DBusString      *file,
               dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
             else
               dbus_set_error (error, DBUS_ERROR_FAILED,
-                              "Error in file %s, line %d, column %d: %s\n",
+                              "Error in file %s, line %lu, column %lu: %s\n",
                               filename,
-                              XML_GetCurrentLineNumber (expat),
-                              XML_GetCurrentColumnNumber (expat),
+                              /* The XML_Size type varies according to
+                               * build options, so cast to something we can
+                               * cope with. */
+                              (unsigned long) XML_GetCurrentLineNumber (expat),
+                              (unsigned long) XML_GetCurrentColumnNumber (expat),
                               XML_ErrorString (e));
           }
 
