@@ -128,6 +128,11 @@ signal_handler (int sig)
           }
       }
       break;
+
+    default:
+      /* can't happen unless this signal handler gets used for a wrong
+       * signal, but keep -Wswitch-default happy */
+      break;
     }
 }
 #endif /* DBUS_UNIX */
@@ -184,7 +189,7 @@ introspect (void)
   if (!_dbus_string_init (&xml))
     goto oom;
 
-  if (!bus_driver_generate_introspect_string (&xml))
+  if (!bus_driver_generate_introspect_string (&xml, TRUE, NULL))
     {
       _dbus_string_free (&xml);
       goto oom;
