@@ -42,7 +42,17 @@ DBUS_BEGIN_DECLS
  */
 
 typedef struct DBusMessage DBusMessage;
-/** Opaque type representing a message iterator. Can be copied by value, and contains no allocated memory so never needs to be freed and can be allocated on the stack. */
+/**
+ * Opaque type representing a message iterator. Can be copied by value and
+ * allocated on the stack.
+ *
+ * A DBusMessageIter usually contains no allocated memory. However, there
+ * is one special case: after a successful call to
+ * dbus_message_iter_open_container(), the caller is responsible for calling
+ * either dbus_message_iter_close_container() or
+ * dbus_message_iter_abandon_container() exactly once, with the same pair
+ * of iterators.
+ */
 typedef struct DBusMessageIter DBusMessageIter;
 
 /**
@@ -66,6 +76,28 @@ struct DBusMessageIter
   void *pad3;           /**< Don't use this */
 };
 
+/**
+ * A message iterator for which dbus_message_iter_abandon_container_if_open()
+ * is the only valid operation.
+ */
+#define DBUS_MESSAGE_ITER_INIT_CLOSED \
+{ \
+  NULL, /* dummy1 */ \
+  NULL, /* dummy2 */ \
+  0, /* dummy3 */ \
+  0, /* dummy4 */ \
+  0, /* dummy5 */ \
+  0, /* dummy6 */ \
+  0, /* dummy7 */ \
+  0, /* dummy8 */ \
+  0, /* dummy9 */ \
+  0, /* dummy10 */ \
+  0, /* dummy11 */ \
+  0, /* pad1 */ \
+  NULL, /* pad2 */ \
+  NULL /* pad3 */ \
+}
+
 DBUS_EXPORT
 DBusMessage* dbus_message_new               (int          message_type);
 DBUS_EXPORT
@@ -207,6 +239,8 @@ dbus_bool_t dbus_message_get_args_valist      (DBusMessage     *message,
 DBUS_EXPORT
 dbus_bool_t dbus_message_contains_unix_fds    (DBusMessage *message);
 
+DBUS_EXPORT
+void        dbus_message_iter_init_closed        (DBusMessageIter *iter);
 DBUS_EXPORT
 dbus_bool_t dbus_message_iter_init             (DBusMessage     *message,
                                                 DBusMessageIter *iter);
@@ -266,6 +300,10 @@ DBUS_EXPORT
 void        dbus_message_iter_abandon_container  (DBusMessageIter *iter,
                                                   DBusMessageIter *sub);
 
+DBUS_EXPORT
+void        dbus_message_iter_abandon_container_if_open (DBusMessageIter *iter,
+                                                         DBusMessageIter *sub);
+
 DBUS_EXPORT
 void dbus_message_lock    (DBusMessage  *message);
 
