@@ -429,6 +429,10 @@ read_data (DBusBabysitter *sitter,
 
     case READ_STATUS_OK:
       break;
+
+    default:
+      _dbus_assert_not_reached ("invalid ReadStatus");
+      break;
     }
   
   if (got == 1)
@@ -453,6 +457,9 @@ read_data (DBusBabysitter *sitter,
                 return r;
               case READ_STATUS_OK:
                 break;
+              default:
+                _dbus_assert_not_reached ("invalid ReadStatus");
+                break;
               }
             
             if (got == 1)
@@ -513,6 +520,9 @@ read_data (DBusBabysitter *sitter,
                 return r;
               case READ_STATUS_OK:
                 break;
+              default:
+                _dbus_assert_not_reached ("invalid ReadStatus");
+                break;
               }
             
             sitter->grandchild_pid = pid;
