@@ -1380,12 +1380,14 @@ _dbus_lm_strerror(int error_number)
       return "The password does not meet the requirements of the password filter DLLs.";
 #endif
 
-    }
-  msg = strerror (error_number);
-  if (msg == NULL)
-    msg = "unknown";
+      default:
+        msg = strerror (error_number);
 
-  return msg;
+        if (msg == NULL)
+          msg = "unknown";
+
+        return msg;
+    }
 #endif //DBUS_WINCE
 }
 
@@ -1479,6 +1481,21 @@ _dbus_replace_install_prefix (DBusString *path)
 #define DBUS_STANDARD_SESSION_SERVICEDIR "/dbus-1/services"
 #define DBUS_STANDARD_SYSTEM_SERVICEDIR "/dbus-1/system-services"
 
+/**
+ * Returns the standard directories for a session bus to look for
+ * transient service activation files. On Windows, there are none.
+ *
+ * @param dirs the directory list we are returning
+ * @returns #TRUE
+ */
+dbus_bool_t
+_dbus_set_up_transient_session_servicedirs (DBusList  **dirs,
+                                            DBusError  *error)
+{
+  /* Not an error, we just don't have transient session services on Windows */
+  return TRUE;
+}
+
 /**
  * Returns the standard directories for a session bus to look for service
  * activation files
