@@ -27,13 +27,13 @@
 #include <config.h>
 #include "test-utils-glib.h"
 
+#include <errno.h>
 #include <string.h>
 
 #ifdef DBUS_WIN
 # include <io.h>
 # include <windows.h>
 #else
-# include <errno.h>
 # include <signal.h>
 # include <unistd.h>
 # include <sys/types.h>
@@ -95,12 +95,14 @@ spawn_dbus_daemon (const gchar *binary,
     const gchar *configuration,
     const gchar *listen_address,
     TestUser user,
+    const gchar *runtime_dir,
     GPid *daemon_pid)
 {
   GError *error = NULL;
   GString *address;
   gint address_fd;
   GPtrArray *argv;
+  gchar **envp;
 #ifdef DBUS_UNIX
   const struct passwd *pwd = NULL;
 #endif
@@ -149,6 +151,8 @@ spawn_dbus_daemon (const gchar *binary,
 
             break;
 
+          case TEST_USER_ME:
+            /* cannot get here, fall through */
           default:
             g_assert_not_reached ();
         }
@@ -158,6 +162,11 @@ spawn_dbus_daemon (const gchar *binary,
 #endif
     }
 
+  envp = g_get_environ ();
+
+  if (runtime_dir != NULL)
+    envp = g_environ_setenv (envp, "XDG_RUNTIME_DIR", runtime_dir, TRUE);
+
   argv = g_ptr_array_new_with_free_func (g_free);
   g_ptr_array_add (argv, g_strdup (binary));
   g_ptr_array_add (argv, g_strdup (configuration));
@@ -175,7 +184,7 @@ spawn_dbus_daemon (const gchar *binary,
 
   g_spawn_async_with_pipes (NULL, /* working directory */
       (gchar **) argv->pdata,
-      NULL, /* envp */
+      envp,
       G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
 #ifdef DBUS_UNIX
       child_setup, (gpointer) pwd,
@@ -190,6 +199,7 @@ spawn_dbus_daemon (const gchar *binary,
   g_assert_no_error (error);
 
   g_ptr_array_free (argv, TRUE);
+  g_strfreev (envp);
 
   address = g_string_new (NULL);
 
@@ -228,6 +238,7 @@ spawn_dbus_daemon (const gchar *binary,
 gchar *
 test_get_dbus_daemon (const gchar *config_file,
                       TestUser     user,
+                      const gchar *runtime_dir,
                       GPid        *daemon_pid)
 {
   gchar *dbus_daemon;
@@ -296,7 +307,7 @@ test_get_dbus_daemon (const gchar *config_file,
   else
     {
       address = spawn_dbus_daemon (dbus_daemon, arg,
-          listen_address, user, daemon_pid);
+          listen_address, user, runtime_dir, daemon_pid);
     }
 
   g_free (dbus_daemon);
@@ -400,18 +411,39 @@ test_connect_to_bus_as_user (TestMainContext *ctx,
       case TEST_USER_ME:
         return test_connect_to_bus (ctx, address);
 
-      default:
+      case TEST_USER_ROOT:
+      case TEST_USER_MESSAGEBUS:
+      case TEST_USER_OTHER:
         g_test_skip ("setresuid() not available, or unsure about "
             "credentials-passing semantics on this platform");
         return NULL;
+
+      default:
+        g_return_val_if_reached (NULL);
     }
 
 #endif
 }
 
+static void
+pid_died (GPid pid,
+          gint status,
+          gpointer user_data)
+{
+  gboolean *result = user_data;
+
+  g_assert (result != NULL);
+  g_assert (!*result);
+  *result = TRUE;
+}
+
 void
 test_kill_pid (GPid pid)
 {
+  gint died = FALSE;
+
+  g_child_watch_add (pid, pid_died, &died);
+
 #ifdef DBUS_WIN
   if (pid != NULL)
     TerminateProcess (pid, 1);
@@ -419,6 +451,9 @@ test_kill_pid (GPid pid)
   if (pid > 0)
     kill (pid, SIGTERM);
 #endif
+
+  while (!died)
+    g_main_context_iteration (NULL, TRUE);
 }
 
 static gboolean
@@ -472,3 +507,95 @@ test_progress (char symbol)
   if (g_test_verbose () && isatty (1))
     g_print ("%c", symbol);
 }
+
+/*
+ * Delete @path, with a retry loop if the system call is interrupted by
+ * an async signal. If @path does not exist, ignore; otherwise, it is
+ * required to be a non-directory.
+ */
+void
+test_remove_if_exists (const gchar *path)
+{
+  while (g_remove (path) != 0)
+    {
+      int saved_errno = errno;
+
+      if (saved_errno == ENOENT)
+        return;
+
+#ifdef G_OS_UNIX
+      if (saved_errno == EINTR)
+        continue;
+#endif
+
+      g_error ("Unable to remove file \"%s\": %s", path,
+               g_strerror (saved_errno));
+    }
+}
+
+/*
+ * Delete empty directory @path, with a retry loop if the system call is
+ * interrupted by an async signal. @path is required to exist.
+ */
+void
+test_rmdir_must_exist (const gchar *path)
+{
+  while (g_remove (path) != 0)
+    {
+      int saved_errno = errno;
+
+#ifdef G_OS_UNIX
+      if (saved_errno == EINTR)
+        continue;
+#endif
+
+      g_error ("Unable to remove directory \"%s\": %s", path,
+               g_strerror (saved_errno));
+    }
+}
+
+/*
+ * Delete empty directory @path, with a retry loop if the system call is
+ * interrupted by an async signal. If @path does not exist, ignore.
+ */
+void
+test_rmdir_if_exists (const gchar *path)
+{
+  while (g_remove (path) != 0)
+    {
+      int saved_errno = errno;
+
+      if (saved_errno == ENOENT)
+        return;
+
+#ifdef G_OS_UNIX
+      if (saved_errno == EINTR)
+        continue;
+#endif
+
+      g_error ("Unable to remove directory \"%s\": %s", path,
+               g_strerror (saved_errno));
+    }
+}
+
+/*
+ * Create directory @path, with a retry loop if the system call is
+ * interrupted by an async signal.
+ */
+void
+test_mkdir (const gchar *path,
+            gint mode)
+{
+  while (g_mkdir (path, mode) != 0)
+    {
+      int saved_errno = errno;
+
+#ifdef G_OS_UNIX
+      if (saved_errno == EINTR)
+        continue;
+#endif
+
+      g_error ("Unable to create directory \"%s\": %s", path,
+               g_strerror (saved_errno));
+    }
+}
